
import processing.serial.*;
import ddf.minim.*;

final int   NUMPLAYERS = 4 ;
final float STRETCHFAC = 1.0 + NUMPLAYERS*(1/256.0);

Minim minim ;
AudioSnippet klaxon ;

PShape picture ;

int[]   yPos ;
int[]   xPos ;
int     maxY ;
int     minY ;
int     maxX ;
int     minX ;
int     dotSize ;
boolean ropeOK ;

color dotColor = color(0) ;
color linColor = color(255) ;
color strColor = #A0522D ;
color badColor = #0000FF ;
int   minStress ;
float maxStress ;

void setup() {
  size(600, 600 /*, P3D */);
  xPos = new int[NUMPLAYERS] ;
  yPos = new int[NUMPLAYERS] ;
  maxY = (int) (0.125*height) ;
  minY = (int) (0.875*height) ;
  dotSize = (int) (min(width, height)/24) ;
  for (int i=0; i<NUMPLAYERS; ++i) {
    yPos[i] = minY ;
    xPos[i] = (int) ((2*i+1)*width/(2*NUMPLAYERS)) ;
  }
  maxX = xPos[NUMPLAYERS-1] ;
  minX = xPos[0] ;
  ellipseMode(CENTER) ;
  minStress = xPos[NUMPLAYERS-1] - xPos[0] ;
  maxStress = STRETCHFAC*minStress ;
  colorMode(HSB, 1.0) ;
  ropeOK = true ;
  
  minim = new Minim(this) ;
  klaxon = minim.loadSnippet("KLAXON.mp3") ;

}

void draw() {
  if (ropeOK) {
    float presStress = 0 ;
    for (int i=1; i<NUMPLAYERS; ++i) {
      presStress += dist(xPos[i-1], yPos[i-1], xPos[i], yPos[i]) ;
    }
    float presHue = map(constrain(presStress, minStress, maxStress),
                        minStress, maxStress, 0.3333333333, 0.0) ;
    background(color(presHue, 1.0, 1.0));
    if (presStress > maxStress) {
      ropeOK = false ;
      klaxon.loop() ;
    }
  } else {
    background(badColor) ;
  }
  strokeWeight(1) ;
  stroke(linColor) ;
  line(minX, minY, maxX, minY) ;
  line(minX, maxY, maxX, maxY) ;
  fill(dotColor) ;
  noStroke() ;
  for (int i=0; i<NUMPLAYERS; ++i) {
    ellipse(xPos[i], yPos[i], dotSize, dotSize) ;
  }
  if (ropeOK) {
    strokeWeight(5) ;
    stroke(strColor) ;
    for (int i=1; i<NUMPLAYERS; ++i) {
      line(xPos[i-1], yPos[i-1], xPos[i], yPos[i]) ;
    }
  }
}

int selectedDot = -1 ;
void mouseDragged() {
  int   closestDot   = -1; 
  float closestDist  = width + height ;
  for (int i=0; i<NUMPLAYERS; ++i) {
    float newDist = dist(xPos[i], yPos[i], mouseX, mouseY) ;
    if (newDist < closestDist) {
      closestDot = i ;
      closestDist = newDist ;
    }
  }
  if (closestDist < dotSize) { 
    selectedDot = closestDot ;
    yPos[closestDot] = max(maxY,min(minY,mouseY)) ;
  }
  if (!ropeOK) {
    int i ;
    for (i=0; i<NUMPLAYERS && yPos[i] == minY; ++i) ;
    if (i==NUMPLAYERS) {
      klaxon.pause() ;
      klaxon.rewind() ;
      ropeOK = true ; 
    }
  }
}

void stop()
{
  klaxon.close() ;
  minim.stop();
  super.stop();
}

// To make it harder --- or easier
final float CHEATCHANGEFAC = 0.1 ;
void keyPressed() {
  if (key == ENTER) {
    // Refactoring needed here
    for (int i=0; i<NUMPLAYERS; ++i) {
      yPos[i] = minY ;
      xPos[i] = (int) ((2*i+1)*width/(2*NUMPLAYERS)) ;
    }
    if (!ropeOK) {
      ropeOK = true ;
      klaxon.pause() ;
      klaxon.rewind() ;
    }
  } else if (key == CODED) {
    if (keyCode == UP || keyCode == RIGHT) {
      maxStress += (maxStress-minStress)*CHEATCHANGEFAC ;
    } else if (keyCode == DOWN || keyCode == LEFT) {
      maxStress -= (maxStress-minStress)*CHEATCHANGEFAC ;
    }
  }
  redraw() ;
}

